use rand::thread_rng;
use rand::Rng;
use rayon::prelude::*;

pub fn rand_array(n: usize, m: u64) -> Vec<u64> {
    let mut rng = thread_rng();
    std::iter::repeat_with(|| rng.gen_range(2..m))
        .take(n)
        .collect()
}

// Functions that sum the values in a vector, using various methods.

pub fn sum_with_index(vec: &Vec<u64>) -> u64 {
    let mut sum = 0;
    for i in 0..vec.len() {
        sum += &vec[i];
    }
    sum
}

pub fn sum_with_loop(vec: &Vec<u64>) -> u64 {
    let mut sum = 0;
    for v in vec {
        sum += v;
    }
    sum
}

pub fn sum_with_iter(vec: &Vec<u64>) -> u64 {
    vec.iter().sum()
}

pub fn sum_with_par_iter(vec: &Vec<u64>) -> u64 {
    vec.par_iter().sum()
}

// Functions that calculate the (total) number of factors in a vector.
// In this code, there is a little more work for each element than a simple sum.

fn number_of_factors(n: &u64) -> u64 {
    (2..=(*n)).into_iter().filter(|i| n % i == 0).count() as u64
}

pub fn factors_with_index(vec: &Vec<u64>) -> u64 {
    let mut sum = 0;
    for i in 0..vec.len() {
        sum += number_of_factors(&vec[i]);
    }
    sum
}

pub fn factors_with_loop(vec: &Vec<u64>) -> u64 {
    let mut sum = 0;
    for v in vec {
        sum += number_of_factors(v);
    }
    sum
}

pub fn factors_with_iter(vec: &Vec<u64>) -> u64 {
    vec.iter().map(number_of_factors).sum()
}

pub fn factors_with_par_iter(vec: &Vec<u64>) -> u64 {
    vec.par_iter().map(number_of_factors).sum()
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn rand_array_tests() {
        assert_eq!(rand_array(123, 456).len(), 123);
    }
    #[test]
    fn num_factors_tests() {
        assert_eq!(number_of_factors(&5), 1);
        assert_eq!(number_of_factors(&24), 7);
    }
    #[test]
    fn sums_tests() {
        let v = Vec::from([3, 4, 5, 6]);
        assert_eq!(sum_with_index(&v), 18);
        assert_eq!(sum_with_loop(&v), 18);
        assert_eq!(sum_with_iter(&v), 18);
        assert_eq!(sum_with_par_iter(&v), 18);
    }
}
